package de.dhsn.oop.ui;

import de.dhsn.oop.data.TextTodoItem;
import de.dhsn.oop.data.TodoItem;
import de.dhsn.oop.data.TodoList;
import de.dhsn.oop.ui.UIOverrides.FlatButton;
import de.dhsn.oop.ui.UIOverrides.FlatLabel;
import de.dhsn.oop.ui.UIOverrides.FlatPanel;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;

public class TodoListView extends JFrame {
    private FlatLabel title;
    private FlatButton addButton;
    private FlatButton deleteButton;
    private TodoList list;
    private FlatPanel scrollContent;
    private JScrollPane scrolPane;
    MainWindow mainWindow;

    public TodoListView(TodoList list, MainWindow mw) {
        super();
        mainWindow = mw;
        this.list = list;

        setMinimumSize(new Dimension(100, 100));
        setSize(new Dimension(100, 300));
        Container cp = getContentPane();
        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        title = new FlatLabel(list.getListTitle());
        cp.add(title, c);
        c.gridx++;

        addButton = new FlatButton("+");
        addButton.addActionListener(this::addTodoClicked);
        cp.add(addButton, c);
        c.gridx++;

        deleteButton = new FlatButton("\uD83D\uDEAE");
        deleteButton.addActionListener(this::deleteButtonClicked);
        cp.add(deleteButton, c);
        c.gridx = 0;
        c.gridy++;
        c.gridwidth = 3;
        c.weightx = 1;
        c.weighty = 1;
        c.fill = GridBagConstraints.BOTH;
        c.gridheight = 5;

        // scrolling content panel
        //not setting a size results in immediate collapsing of the lsit upon resizing the window
        scrollContent = new FlatPanel();

        buildScrollPanel();

        cp.add(scrolPane, c);
        pack();
        setVisible(true);
    }

    public void buildScrollPanel(){
        scrollContent = new FlatPanel();
        scrollContent.removeAll();
        scrollContent.setMinimumSize(new Dimension(50,50));
        scrollContent.setLayout(new BoxLayout(scrollContent, BoxLayout.Y_AXIS));
        for (TodoItem t: list.getList())
            scrollContent.add(new TodoItemPanel(t, list, this));
        scrolPane = new JScrollPane(scrollContent);
        revalidate();
        repaint();
    }

    public void forceRerender(){
        mainWindow.restartList(list);
        setVisible(false);
        dispose();
    }

    public void addTodoClicked(ActionEvent e){
        new NewTodoItemView(this);
    }

    public void deleteButtonClicked(ActionEvent e){
        mainWindow.deleteList(list);
        setVisible(false);
        dispose();
    }

    public void addCallback(TextTodoItem item){
        list.addTodo(item);
        // todo fühlt sich nach nem reudigen hack an
        forceRerender();
    }
}
